@@ -7,9 +7,9 @@ from borgmatic.execute import execute_command
 logger = logging.getLogger(__name__)
 
 
-def make_prune_flags(storage_config, retention_config, local_borg_version):
+def make_prune_flags(config, local_borg_version):
     '''
-    Given a retention config dict mapping from option name to value, transform it into an sequence of
+    Given a configuration dict mapping from option name to value, transform it into an sequence of
     command-line flags.
 
     For example, given a retention config of:
@@ -23,12 +23,12 @@ def make_prune_flags(storage_config, retention_config, local_borg_version):
             ('--keep-monthly', '6'),
         )
     '''
-    config = retention_config.copy()
-    prefix = config.pop('prefix', None)
-
     flag_pairs = (
-        ('--' + option_name.replace('_', '-'), str(value)) for option_name, value in config.items()
+        ('--' + option_name.replace('_', '-'), str(value))
+        for option_name, value in config.items()
+        if option_name.startswith('keep_') and option_name != 'keep_exclude_tags'
     )
+    prefix = config.get('prefix')
 
     return tuple(element for pair in flag_pairs for element in pair) + (
         (
@@ -39,8 +39,8 @@ def make_prune_flags(storage_config, retention_config, local_borg_version):
         if prefix
         else (
             flags.make_match_archives_flags(
-                storage_config.get('match_archives'),
-                storage_config.get('archive_name_format'),
+                config.get('match_archives'),
+                config.get('archive_name_format'),
                 local_borg_version,
             )
         )
@@ -50,40 +50,43 @@ def make_prune_flags(storage_config, retention_config, local_borg_version):
 def prune_archives(
     dry_run,
     repository_path,
-    storage_config,
-    retention_config,
+    config,
     local_borg_version,
+    prune_arguments,
+    global_arguments,
     local_path='borg',
     remote_path=None,
-    stats=False,
-    list_archives=False,
 ):
     '''
-    Given dry-run flag, a local or remote repository path, a storage config dict, and a
-    retention config dict, prune Borg archives according to the retention policy specified in that
-    configuration.
+    Given dry-run flag, a local or remote repository path, and a configuration dict, prune Borg
+    archives according to the retention policy specified in that configuration.
     '''
     borgmatic.logger.add_custom_log_levels()
-    umask = storage_config.get('umask', None)
-    lock_wait = storage_config.get('lock_wait', None)
-    extra_borg_options = storage_config.get('extra_borg_options', {}).get('prune', '')
+    umask = config.get('umask', None)
+    lock_wait = config.get('lock_wait', None)
+    extra_borg_options = config.get('extra_borg_options', {}).get('prune', '')
 
     full_command = (
         (local_path, 'prune')
-        + make_prune_flags(storage_config, retention_config, local_borg_version)
+        + make_prune_flags(config, local_borg_version)
         + (('--remote-path', remote_path) if remote_path else ())
         + (('--umask', str(umask)) if umask else ())
+        + (('--log-json',) if global_arguments.log_json else ())
         + (('--lock-wait', str(lock_wait)) if lock_wait else ())
-        + (('--stats',) if stats and not dry_run else ())
+        + (('--stats',) if prune_arguments.stats and not dry_run else ())
         + (('--info',) if logger.getEffectiveLevel() == logging.INFO else ())
-        + (('--list',) if list_archives else ())
+        + flags.make_flags_from_arguments(
+            prune_arguments,
+            excludes=('repository', 'stats', 'list_archives'),
+        )
+        + (('--list',) if prune_arguments.list_archives else ())
         + (('--debug', '--show-rc') if logger.isEnabledFor(logging.DEBUG) else ())
         + (('--dry-run',) if dry_run else ())
         + (tuple(extra_borg_options.split(' ')) if extra_borg_options else ())
         + flags.make_repository_flags(repository_path, local_borg_version)
     )
 
-    if stats or list_archives:
+    if prune_arguments.stats or prune_arguments.list_archives:
         output_log_level = logging.ANSWER
     else:
         output_log_level = logging.INFO
@@ -91,6 +94,7 @@ def prune_archives(
     execute_command(
         full_command,
         output_log_level=output_log_level,
+        extra_environment=environment.make_environment(config),
         borg_local_path=local_path,
-        extra_environment=environment.make_environment(storage_config),
+        borg_exit_codes=config.get('borg_exit_codes'),
     )
